<?php
/**
 * @package NantiAja Indonesia
 * @version 1.1.2
 */
/**
 * Plugin Name: NantiAja Indonesia
 * Plugin URI: https://wordpress.org/plugins/nantiaja-indonesia/
 * Description: A payment gateway for WooCommerce.
 * Version: 1.1.2
 * Author: NantiAja
 * Author URI: https://nantiaja.com/
 * Text Domain: nantiaja-indonesia
 * WC requires at least: 2.2.0
 * WC tested up to: 5.6.0
 */
/**
 * Copyright (c) 2021 NantiAja Indonesia.
 *
 * The name of the NantiAja may not be used to endorse or promote products derived from this
 * software without specific prior written permission. THIS SOFTWARE IS PROVIDED ``AS IS'' AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * CSS
 */
function nantiaja_checkout_css() {
	wp_register_style('nantiaja_checkout_css', plugins_url('assets/css/checkout.css', __FILE__));
	wp_enqueue_style('nantiaja_checkout_css');
}
function nantiaja_pdp_css() {
	wp_register_style('nantiaja_pdp_css', plugins_url('assets/css/pdp.css', __FILE__));
	wp_enqueue_style('nantiaja_pdp_css');
}
add_action('wp_enqueue_scripts', 'nantiaja_checkout_css');
add_action('wp_enqueue_scripts', 'nantiaja_pdp_css');

/**
 * Javascript
 */
function nantiaja_variables_js() {
	$nantiaja = new WC_NantiAja();
	// plugin should be enabled
	if ( $nantiaja->enabled != 'yes' ) return;
	$theme = strtolower( preg_replace('/[^a-zA-Z0-9]+/', '-', wp_get_theme() ));
	$version = wp_get_theme()->version;
	$theme_color = isset($nantiaja->theme_color) ? $nantiaja->theme_color : 'light';
	echo
		'<script>
			let nantiaja_theme_name = "'. $theme .'",
				nantiaja_theme_version = "'. $version .'"
				nantiaja_theme_color = "'. $theme_color .'"
		</script>';
}
function nantiaja_checkout_js()
{
	if ( !is_checkout() ) return;

	wp_enqueue_script( 'nantiaja_checkout_js', plugins_url('/assets/js/nantiaja_checkout.js', __FILE__), [], '0', true );
	$theme = strtolower( preg_replace('/[^a-zA-Z0-9]+/', '-', wp_get_theme() ));
	$version = wp_get_theme()->version;
	echo
		'<script>
			let nantiaja_current_page = "checkout"
		</script>';

	$url = 'https://static.nantiaja.com/wc-snippet.js' . ('?v=' . rand(10,100)); 
	
	wp_register_script( 'nantiaja_pdp_js', $url, [], null);
	wp_enqueue_script( 'nantiaja_pdp_js' );
}
function nantiaja_pdp_js() {
	// display on product page only
	if ( !is_product() ) return;
	$nantiaja = new WC_NantiAja();
	// plugin should be enabled
	if ( $nantiaja->enabled != 'yes' ) return;
	// option should be enabled
	if ( $nantiaja->enable_pdp != 'yes' ) return;
	// product
	$product = wc_get_product( get_the_ID() );

	if( (int) $product->is_type( 'variable' ) == 1 ) {
		$amount	=  $product->get_variation_sale_price( 'max', true );
	} else {
		$amount = $product->get_price();
	}
	
	$productAmount = $product->get_price();
	echo
		'<script>
			let nantiaja_amount = "'. $amount .'",
				nantiaja_product_sale_amount = "'. $productAmount .'",
				nantiaja_get_regular_price = "'. $product->get_regular_price() .'",
				nantiaja_get_sale_price = "'. $product->get_sale_price() .'"
		</script>';

	$url = 'https://static.nantiaja.com/wc-snippet.js' . ('?v=' . rand(10,100)); 

	wp_register_script( 'nantiaja_pdp_js', $url, [], null);
	wp_enqueue_script( 'nantiaja_pdp_js' );
}
// Checkout
add_action( 'wp_enqueue_scripts', 'nantiaja_variables_js', 99999 );
add_action( 'wp_enqueue_scripts', 'nantiaja_checkout_js', 99999 );
add_action( 'wp_enqueue_scripts', 'nantiaja_pdp_js', 99999 );

/**
 * Registers to WooCommerce payment gateway
 */
add_filter('woocommerce_payment_gateways', 'nantiaja_class');
function nantiaja_class( $gateways ) {
	$gateways[] = 'WC_NantiAja';
	return $gateways;
}

/**
 * Check if WooCommerce Plugin is installed first
 */
function nantiaja_check_wc() {
    if (!defined('WC_VERSION')) {
        // no woocommerce
		echo "<div>
             <p style='font-weight: 400;
    			font-size: 14px;
    			line-height: 1.5;
				color: #444;
    			font-family: -apple-system, sans-serif;'
				>Please activate WooCommerce.</p>
         	</div>";
		exit;
    } else {
		add_action( 'plugins_loaded', 'nantiaja_init_class' );
	}
}

add_action( 'plugins_loaded', 'nantiaja_init_class' );
register_activation_hook( __FILE__, 'nantiaja_check_wc' );

function nantiaja_init_class() {
	class WC_NantiAja extends WC_Payment_Gateway
	{
		/**
		 * Class constructor
		 * 
		 */
		public function __construct() {
			global $woocommerce;

			$this->id = 'nantiaja'; // payment gateway plugin ID
			$this->icon = plugins_url('/assets/logo.png', __FILE__);

			$this->has_fields = true; // To be shown on the checkout
			$this->method_title = 'NantiAja Indonesia';
			$this->method_description = 'Allow customers to conveniently checkout directly with NantiAja.';
			$this->order_button_text = 'Bayar';
		 
			// define and load settings
			$this->init_form_fields();
			$this->init_settings();
			$this->title = 'PayLater pakai NantiAja';

			$totAmount = @$woocommerce->cart->total / 4;
			$formattedAmount = $totAmount;
			$this->description = '
				<div class="container-nantiaja">
					<h4>Bagi Pembayaran Jadi 4, Gratis</h4>
					<hr>
					<p>Nikmati pembayaran tanpa bunga</p>
				</div>';

			$this->enabled = $this->get_option( 'enabled' );
			$this->environment = $this->get_option('environment');

			// Production
			$this->store_id = $this->get_option('store_id');
			$this->secret_key = $this->get_option('secret_key');

			// Sandbox
			$this->sb_store_id = $this->get_option('sb_store_id');
			$this->sb_secret_key = $this->get_option('sb_secret_key');

			// PDP
			$this->enable_pdp = $this->get_option('enable_pdp');
			
			// Theme Color
			$this->theme_color = $this->get_option('theme_color');
	 
			// This action hook saves the settings you input inside the admin dashbaord
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
			// Callback handler
			add_action( 'woocommerce_api_nantiaja-callback', array( $this, 'webhook' ) );
		}

		/**
		* Plugin options
		*/
		public function init_form_fields(){
			$this->form_fields = array(
				'enabled' => array(
					'title'       => 'Enable/Disable',
					'label'       => 'Enable NantiAja Checkout',
					'type'        => 'checkbox',
					'description' => '',
					'default'     => 'no'
				),
				'environment' => [
					'title'       => 'Select environment',
					'label'       => 'Enable NantiAja Gateway',
					'type'        => 'select',
					'description' => '',
					'options' 	=> [
						'sandbox' 	 => 'Sandbox',
						'production' => 'Production',
					]
				],
				'store_id' => [
					'title' => 'Production Public Key',
					'type'	=> 'text'
				],
				'secret_key' => [
					'title' => 'Production Token',
					'type'	=> 'password'
				],
				'sb_store_id' => [
					'title' => 'Sandbox Public Key',
					'type'	=> 'text'
				],
				'sb_secret_key' => [
					'title' => 'Sandbox Token',
					'type'	=> 'password'
				],
				'enable_pdp' => [
					'title' 		=> 'Enable PDP',
					'label'			=> 'This will enable the PDP snippet on your single product page',
					'default'		=> 'no',
					'type'			=> 'checkbox'
				],
				'theme_color' => [
					'title'       => 'Select theme color',
					'label'       => 'Select theme color',
					'type'        => 'select',
					'default'		=> 'light',
					'description' => '',
					'options' 	=> [
						'light' 	 => 'Light',
						'dark' => 'Dark',
					]
				],
			);
		}

		/**
		* Custom form
		*/
		public function payment_fields()
		{
			echo wpautop( wp_kses_post( $this->description ) );
		}

		/**
		  * Processing the payment
		  */
		public function process_payment( $order_id )
		{
			global $woocommerce;

			$nantiaja = new WC_NantiAja();
			$order = new WC_Order( $order_id );

			$totAmount = round($woocommerce->cart->total);
			$cart = array();
			$order_items = $order->get_items();
			$order_data = $order->get_data();

			$send = array(
				"total" => $totAmount,
				"orderId" => (string) $order_id,
				"redirectURL" => $this->get_return_url($order),
				"failedURL" => wc_get_checkout_url(),
				"detail" => []
			);

			foreach( $order_items as $product ) {
				$terms = get_the_terms( $product['product_id'], 'product_cat' );
				$cat = $terms[0] ? $terms[0]->slug : '';
				
				$product_variation_id = $product['variation_id'];
				if ($product_variation_id) { // IF Order Item is Product Variantion then get Variation Data instead
					$products = wc_get_product($product['variation_id']);
				} else {
					$products = wc_get_product($product['product_id']);
				}
				if ($products) { // Product might be deleted and not exist anymore    
					$sku = $products->get_sku() != '' ? $products->get_sku() : 'n/a';                 
				}

				$item['sku'] = $sku;
				$item['name'] = $product['name'];
				$item['category'] = $cat;
				$item['price'] = $product['total'];
				$item['quantity'] = $product['qty'];
				$item['discount'] = $product['discount'] ? $product['discount'] : 0;
				array_push($cart,$item);
			}	

			$shipping_country = $order->get_shipping_country();
			$shipping_state = $order->get_shipping_state() != '' ? $order->get_shipping_state() : '';
			$shipping_province = $shipping_state != '' ? WC()->countries->get_states( $shipping_country )[$shipping_state] : '';

			$billing_country = $order->get_billing_country();
			$billing_state = $order->get_billing_state() != '' ? $order->get_billing_state() : '' ;
			$billing_province = $billing_state != '' ? WC()->countries->get_states( $billing_country )[$billing_state] : '';

			$send['detail']['items'] = $cart;
			$send['detail']['shipping'] = [
				"street" => $order->get_shipping_address_1() != '' ? $order->get_shipping_address_1() . ' ' . $order->get_shipping_address_2() : 'n/a',
				"address" => $order->get_shipping_address_1() != '' ? $order->get_shipping_address_1() . ' ' . $order->get_shipping_address_2() : 'n/a',
				"city" => $order->get_shipping_city() != '' ? $order->get_shipping_city() : 'n/a',
				"province" => $shipping_province != '' ? $shipping_province : 'n/a',
				"state" => $shipping_province != '' ? $shipping_province : 'n/a',
				"postalCode" => $order->get_shipping_postcode() != '' ? $order->get_shipping_postcode() : 'n/a'
			];
			$send['detail']['total'] = $totAmount;
			$send['detail']['billingDetail'] = [
				"name" => $order->get_billing_first_name() ? $order->get_billing_first_name() . ' ' . $order->get_billing_last_name() : 'n/a',
				"email" => $order->get_billing_email() ? $order->get_billing_phone() : 'n/a',
				"phone" => $order->get_billing_phone() ? $order->get_billing_phone() : 'n/a',
				"address" => [
					"street" => $order->get_billing_address_1() ? $order->get_billing_address_1() . ' ' . $order->get_billing_address_2() : 'n/a',
					"address" => $order->get_billing_address_1() ? $order->get_billing_address_1() . ' ' . $order->get_billing_address_2() : 'n/a',
					"city" => $order->get_billing_city() ? $order->get_billing_city() : 'n/a',
					"province" => $billing_province != '' ? $billing_province : 'n/a',
					"state" => $billing_province != '' ? $billing_province : 'n/a',
					"postalCode" => $order->get_billing_postcode() ? $order->get_billing_postcode() : 'n/a'
				]
			];

			// print_r($send);

			$storeId = ($nantiaja->environment == 'sandbox') ? $nantiaja->sb_store_id : $nantiaja->store_id;
			$secretKey = ($nantiaja->environment == 'sandbox') ? $nantiaja->sb_secret_key : $nantiaja->secret_key;
			
			$chosenEnvAPI = ( $nantiaja->environment != 'production' ) ? "sb-" : "";
			$chosenEnvPortal = ( $nantiaja->environment != 'production' ) ? "sb-" : "";

			$url = "https://{$chosenEnvAPI}api.nantiaja.com/api/transaction/v1/authentication";

			$args = [
				"body" => json_encode($send),
				"headers" => [
					"Content-Type" => "application/json",
					"Authorization" => "Basic " . base64_encode( $storeId . ":" . $secretKey )
				]
			];

			$response = wp_remote_post( $url, $args );
			$response = json_decode($response["body"]);
			// var_dump($response);
			$response = $response->authentication_url;

			$resultResponse = [];

			if ( $response == "" ) {
				$resultResponse = [
					"result" => "failure",
					"messages" => "Error!",
				];
			}
			else {
				$resultResponse = [
					"result" => "success",
					"redirect" => $response,
				];
			}
			return $resultResponse;
		}

		private function getUserIpAddr()
		{
			if ( !empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
				//ip from share internet
				$ip = $_SERVER['HTTP_CLIENT_IP'];
			} elseif ( !empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
				//ip pass from proxy
				$ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
			} else {
				$ip = $_SERVER['REMOTE_ADDR'];
			}
			return $ip;
		}

		/**
		 * Webhook for payment completion
		 * 
		 * This will accept transactionTime, statusCode, orderId, transactionStatus and total
		 */
		/**
		 * Webhook for payment completion
		 * 
		 * This will accept transactionTime, statusCode, orderId, transactionStatus and total
		 */
		public function webhook()
		{
			header('Content-Type: application/json');
			$payload = json_decode(file_get_contents("php://input"), true);
			$wow = isset( $payload['wow'] ) ? $payload['wow'] : '';

			$nantiaja = new WC_NantiAja();
			$orderId = $payload['orderId'];
			$order = new WC_Order( $orderId );
			$order->add_order_note( $payload ); // debugging purposes

			// Callback to check plugin is enabled or not
			if ( !empty( $wow ) ) {
				$gateways = WC()->payment_gateways->get_available_payment_gateways();
				if($gateways['nantiaja']->enabled =="yes") {
					echo "yes";
				} else {
					echo "no";
				}
				die();
			} else {
				$send = array(
					"orderId" => (string) $orderId
				);
				
				$chosenEnvAPI = ( $nantiaja->environment != 'production' ) ? "sb-" : "";
				$chosenEnvPortal = ( $nantiaja->environment != 'production' ) ? "sb-" : "";

				$url = "https://{$chosenEnvAPI}api.nantiaja.com/api/transaction/v1/order-validator";

				$args = [
					"body" => json_encode($send),
					"headers" => [
						"Content-Type" => "application/json",
					]
				];

				$response = wp_remote_post( $url, $args );
				$response = json_decode($response["body"]);
				// var_dump($response);

				if ( $response ) {
					$order->payment_complete();
					// reduce order stocks
					// check if order status is not "processing"
					if($order->get_status() != "processing") {
						wc_reduce_stock_levels( $orderId );	
					}

					echo json_encode([
						"status" => 200,
						"result" => "success",
						"messages" => "Transaction success",
					]);
					header('Content-Type: application/json');
					http_response_code(200);
					die();
				} else {
					$order->add_order_note( 'Failed IP check' );
					echo json_encode([
						"status" => 400,
						"result" => "failure",
						"messages" => "Failed IP check: " . $ip,
					]);
					header('Content-Type: application/json');
					http_response_code(400);
					die();
				}
			}
		}

		/**
		 * Additional options
		 */
		public function admin_options() {
			?>
			<h2><?php _e('NantiAja Checkout','woocommerce'); ?></h2>
			<p>Allow customers to conveniently checkout directly with NantiAja.</p>
			<table class="form-table">
			<?php $this->generate_settings_html(); ?>
			</table>
			<?php
		}

	    /**
	     * Processes and saves options.
	     * If there is an error thrown, will continue to save and validate fields, but will leave the erroring field out.
	     *
	     * Added filter to invalidate admin to save changes based on the condition per environment.
	     *
	     * @return bool was anything saved?
	     */
	    public function process_admin_options() {
	        $this->init_settings();

	        $post_data = $this->get_post_data();

	        $environment = sanitize_text_field($_POST['woocommerce_nantiaja_environment']);

	        // Production
	        $prod_store_id = sanitize_text_field($_POST['woocommerce_nantiaja_store_id']);
	        $prod_secret_key = sanitize_text_field($_POST['woocommerce_nantiaja_secret_key']);
	        // Sandbox
	        $sandbox_store_id = sanitize_text_field($_POST['woocommerce_nantiaja_sb_store_id']);
	        $sandbox_secret_key = sanitize_text_field($_POST['woocommerce_nantiaja_sb_secret_key']);

	        if ( $environment == 'production' ) {
		        if ( empty( $prod_store_id ) ) {
		        	WC_Admin_Settings::add_error( 'Error: Production Store ID should not be empty!' );
		        	return false;
		        }
		        if ( empty( $prod_secret_key ) ) {
		        	WC_Admin_Settings::add_error( 'Error: Production Secret key should not be empty!' );
		        	return false;
		        }
	        } else {
		        if ( empty( $sandbox_store_id ) ) {
		        	WC_Admin_Settings::add_error( 'Error: Sandbox Store ID should not be empty!' );
		        	return false;
		        }
		        if ( empty( $sandbox_secret_key ) ) {
		        	WC_Admin_Settings::add_error( 'Error: Sandbox Secret key should not be empty!' );
		        	return false;
		        }
	        }
      
	        foreach ( $this->get_form_fields() as $key => $field ) {
	            if ( 'title' !== $this->get_field_type( $field ) ) {
	                try {
	                    $this->settings[ $key ] = $this->get_field_value( $key, $field, $post_data );
	                } catch ( Exception $e ) {
	                	WC_Admin_Settings::add_error( $e->getMessage() );
	                }
	            }
	        }

	        return update_option( $this->get_option_key(), apply_filters( 'woocommerce_settings_api_sanitized_fields_' . $this->id, $this->settings ), 'yes' );
	    }
	}
}
